#include <module.h>

static inline void
module_load_decl (module_t * module, Agnode_t * object)
{

  Agnode_t *type;
  type = resolve_path (object, 1, "cInstantiatedForwardType");
  if (type)
    {
      var_t *var = var_new (object);
      assert (var);

      varn_t *node = NULL;
      linkedlist_t *list = NULL;

      char *str = var->type->sctype;

      assert (str);

      if (!strcmp (str, "sc_in"))
	list = module->input;
      else if (!strcmp (str, "sc_out"))
	list = module->output;
      else if (!strcmp (str, "sc_inout"))
	list = module->inout;
      else if (!strcmp (str, "sc_signal"))
	list = module->signals;
      else
	list = module->variables;

      node = list_add (list, varn_t);
      assert (node);
      node->var = var;
      free (str);

      return;
    }

  type = resolve_path (object, 2, "cPointerType", "cForwardType");
  if (type)
    {
      char *str = get_namedattr (type, "name");
      char *obj = get_namedattr (object, "name");
      //varn_t *node = NULL;
      //printf( "%s of %s\n", obj, str );
      instancen_t *node = list_add (module->instances, instancen_t);
      node->instance = instance_new (obj, str);
      free (str);
      free (obj);
      return;
    }

  type = resolve_path (object, 1, "cArrayType");
  if (type)
    {

      var_t *var = var_new (object);
      assert (var);
      assert (var->name);
      char *s = var_str (var);
      assert (s);

      //panic( "stop for debug %s", s );

      varn_t *node = NULL;
      linkedlist_t *list = NULL;

      char *str = var->type->sctype;

      if (!strcmp (str, "sc_in"))
	list = module->input;
      else if (!strcmp (str, "sc_out"))
	list = module->output;
      else if (!strcmp (str, "sc_inout"))
	list = module->inout;
      else if (!strcmp (str, "sc_signal"))
	list = module->signals;
      else
	list = module->variables;

      node = list_add (list, varn_t);
      assert (node);
      node->var = var;
      free (str);

      return;
    }

  type = resolve_path (object, 1, "cForwardType");
  
  if(!type)
    type = resolve_path (object, 1, "cBuiltInType");
  if (type)
    {

      var_t *var = var_new (object);
      assert (var);
      assert (var->name);

      varn_t *node;
      node = list_add (module->variables, varn_t);
      assert (node);
      node->var = var;

      return;
    }

  print_ast (object);
  panic ("can't load object declaration ! ");

}

int
module_complete (module_t * module)
{
  methodn_t *m;
  for_each_list_element (module->methods, m)
  {
    if (m->method->block == NULL){
      debug("can't find implementation of %s", m->method->name);
      return 0;
    }
  }
  return 1;
}


static inline void
module_load (module_t * module)
{

  Agnode_t *class = module->top;

  char *str = get_namedattr (class, "name");
  module->name = str;

  // for each object list declaration inside the class
  Agedge_t *e2;
  for_each_child (e2, class, cObject)
  {
    Agnode_t *object = e2->head;
    module_load_decl (module, object);
  }

  Agedge_t *e3;
  for_each_child (e3, class, cObjectList)
  {
    Agnode_t *objects = e3->head;

    // for each object declaration inside the list
    Agedge_t *e4;
    for_each_child (e4, objects, cObject)
    {
      Agnode_t *object = e4->head;

      module_load_decl (module, object);
    }

    for_each_child (e4, objects, cEnumerationType)
    {
      Agnode_t *object = e4->head;
      enumeration_load (module->enumerations, e4->head, NULL);
    }
  }

  // for each function declaration inside the class
  Agedge_t *f1;
  for_each_child (f1, class, cFunction)
  {
    Agnode_t *function = f1->head;

    char *key = get_namedattr (function, "key");

    str_case (key, constructor)
    {
      method_t *ctor = method_new (function);
      module->ctor = ctor;
    }
    str_else
    {
      methodn_t *m = list_add (module->methods, methodn_t);
      m->method = method_new (function);
      //m->method->sc_method = true;
    }

    free (key);

  }
  
  Agedge_t *e5;
  for_each_child (e5, class, cEnumerationType)
  {
    enumeration_load (module->enumerations, e5->head, NULL);
  }

  for_each_child (e5, class, cClassType)
  {
    //enumeration_load (module->enumerations, e5->head, NULL);
    //panic("hello world");
    char *name = get_namedattr(e5->head, "name");
    //panic("%s", name);
    struct_load(gunit->structs, e5->head, name);
    free(name);
    //assert(gunit->structs);
  }

  return;
}

static linkedlist_t *
walk_sensitivity (method_t * method, Agnode_t * root)
{

  assert (method);

  linkedlist_t *list = NULL;

  Agedge_t *e;
  for_each_child (e, root, cBinaryOperator)
  {
    Agnode_t *child = e->head;
    char *op_str = get_namedattr (child, "op");
    assert (op_str);
    str_case (op_str, <<)
    {
      list = walk_sensitivity (method, child);
    }
    str_elseif (op_str, ())
    {
      Agedge_t *t = first_child (child);
      Agnode_t *b = t->head;
      assert (b);
      assert (!strcmp (get_namedattr (b, "op"), "."));
      Agedge_t *op1 = first_child (b);
      Agedge_t *op2 = second_child (b);
      assert (op1);
      assert (op2);
      char *op1_str = get_namedattr (op1->head, "name");
      char *op2_str = get_namedattr (op2->head, "name");
      strn_t *node;
      str_case (op2_str, pos)
      {
	node = list_add (method->sensitivity_pos, strn_t);
      }
      str_elseif (op2_str, neg)
      {
	node = list_add (method->sensitivity_neg, strn_t);
      }
      str_else
      {
	assert (0);
      }

      assert (node);
      node->str = op1_str;
      free (op2_str);
    }
    str_elseif(op_str, .){
      Agedge_t *t;
      t = first_child (child);
      Agnode_t *op1 = t->head;
      t = second_child(child);
      Agnode_t *op2 = t->head;
      assert (op1);
      assert (op2);
      char *op1_str = get_namedattr (op1, "name");
      char *op2_str = get_namedattr (op2, "name");
      strn_t *node = list_add (method->sensitivity_def, strn_t);
      assert(node);
      asprintf(&node->str, "%s_%s", op1_str, op2_str);;
      free(op1_str);
      free(op2_str);
    }
    str_else
    {
      assert (0);
    }
    free (op_str);
  }

  Agedge_t *s;
  for_each_child (s, root, cNameReference)
  {
    Agnode_t *name = s->head;
    char *name_str = get_namedattr (name, "name");
    assert (name_str);
    str_case (name_str, sensitive)
    {
      free (name_str);
      list = method->sensitivity_def;
      assert (list);
    }
    str_elseif (name_str, sensitive_pos)
    {
      free (name_str);
      list = method->sensitivity_pos;
      assert (list);
    }
    str_elseif (name_str, sensitive_neg)
    {
      free (name_str);
      list = method->sensitivity_neg;
      assert (list);
    }
    str_else
    {
      if (list)
	{
	  strn_t *node = list_add (list, strn_t);
	  assert (node);
	  node->str = name_str;
	}
    }

  }

  assert (list != NULL);

  return list;

}


static inline instance_t *
module_instance (module_t * module, char *name)
{
  instancen_t *node;
  for_each_list_element (module->instances, node)
  {
    if (!strcmp (node->instance->name, name))
      {
	return node->instance;
      }
  }
  return NULL;
}

static inline method_t *
module_method (module_t * module, char *name)
{
  methodn_t *node;
  for_each_list_element (module->methods, node)
  {
    if (!strcmp (node->method->name, name))
      {
	return node->method;
      }
  }
  return NULL;
}

static inline void
module_elaborate (module_t * module)
{
  Agedge_t *e;
  method_t *method = NULL;

  //assert( module->ctor );
  if (!module->ctor)
    {
      panic ("can't find constructor of module %s", module->name);
    }
  assert (module->ctor->block);

  for_each_child (e, module->ctor->block, cBinaryOperator)
  {

    Agnode_t *op = e->head;
    char *str = get_namedattr (op, "op");

    str_case (str, ())
    {
      Agedge_t *child = first_child (op);
      if (node_typeof (child->head, "cBinaryOperator"))
	{
	  char *ins_str;
	  char *port_str;
	  char *child_str = get_namedattr (child->head, "op");
	  str_case (child_str,->)
	  {
	    Agedge_t *c;

	    c = first_child (child->head);
	    ins_str = get_namedattr (c->head, "name");

	    c = second_child (child->head);
	    port_str = get_namedattr (c->head, "name");

	  }
	  str_else
	  {
	    panic ("error\n");
	  }

	  Agnode_t *args =
	    resolve_path (op, 2, "cFunctionArgumentList", "cNameReference");
	  char *args_str = get_namedattr (args, "name");
	  instance_t *ins = module_instance (module, ins_str);
	  instance_map (ins, port_str, args_str);

	  free (port_str);
	  free (ins_str);
	  free (args_str);

	}
      else if (node_typeof (child->head, "cNameReference"))
	{
	  Agnode_t *name = child->head;
	  char *name_str = get_namedattr (name, "name");
	  //assert( name_str );
	  str_case (name_str, SC_METHOD)
	  {
	    Agnode_t *method_node =
	      resolve_path (op, 2, "cFunctionArgumentList", "cNameReference");

	    char *method_name = get_namedattr (method_node, "name");
	    //printf( "\tMETHOD DECLARATION %s\n", method_name );
	    method = module_method (module, method_name);
	    method->sc_method = true;
	    free (method_name);
	  }
	  str_else
	  {
	    panic ("unknow name reference %s", name_str);
	  }

	  free (name_str);
	}
      else
	{
	  panic ("error : unhandled operator in binary operator : [%s]\n",
		 str);
	}

    }
    str_elseif (str, <<)
    {
      //printf( "hello \n" );
      linkedlist_t *list = NULL;
      //walk_sensitivity (op, &list);
      strn_t *node;

      //walk_sensitivity ( method, op, &list);
      assert (method != NULL);
      list = walk_sensitivity (method, op);

    }
    str_elseif (str, =)
    {
      Agedge_t *ch = first_child (op);
      Agnode_t *head = ch->head;

      Agnode_t *n = resolve_path (op, 2, "cNewExpression", "cForwardType");

      if (n)
	{
	  //printf( "of %s \n", get_namedattr( n, "name" ) );
	  char *inst_str = get_namedattr (head, "name");

	  instance_t *i = module_instance (module, inst_str);
	  assert (i);
	  instance_set (i);

	}

    }
    str_else
    {
      panic ("error : unhandled operator in constructor : [%s]\n", str);
      //assert (0);
    }
    free (str);
  }
}

inline module_t *
module_new (Agnode_t * top)
{
  module_t *m = (module_t *) malloc (sizeof (module_t));
  assert (m);

  m->ctor = NULL;
  m->top = top;

  m->input = list_new ();
  m->output = list_new ();
  m->inout = list_new ();
  m->signals = list_new ();
  m->variables = list_new ();
  m->methods = list_new ();
  m->instances = list_new ();
  m->enumerations = list_new ();

  module_load (m);

  module_elaborate (m);

  return m;
}

inline void
module_free (module_t * m)
{
  assert (m);
  free (m);
  return;
}
