#include "crc.h"

void crc::crc_check_clock_event() {
  // Temprary signal to be assigned the new value of crc_result
  sc_bv<16> new_crc_result;
  
  if (! reset_n.read() ) {
    // Shift register initial state, ISO spec, page 31 (mpgaudio.doc)
    new_crc_result = "1111111111111111";

  } else { 
    if (enable.read()) { 
      // G(X) = X^16 + X^15 + X^2 + 17
      // Implementation of Appendix A, page 7, Figure 3-A.9, CRC-Check diagram
      
      // Temporary signal to allow indexing of out signal bits
      sc_bv<16> var_crc_result = crc_result.read();
      
      sc_bit xor_control;
      xor_control = bit_in.read() ^ var_crc_result[0x0F];

      new_crc_result[0x00] = xor_control;
      new_crc_result[0x01] = var_crc_result[0x00];
      new_crc_result[0x02] = var_crc_result[0x01] ^ xor_control;
      new_crc_result[0x03] = var_crc_result[0x02];
      new_crc_result[0x04] = var_crc_result[0x03];
      new_crc_result[0x05] = var_crc_result[0x04];
      new_crc_result[0x06] = var_crc_result[0x05];
      new_crc_result[0x07] = var_crc_result[0x06];
      new_crc_result[0x08] = var_crc_result[0x07];
      new_crc_result[0x09] = var_crc_result[0x08];
      new_crc_result[0x0A] = var_crc_result[0x09];
      new_crc_result[0x0B] = var_crc_result[0x0A];
      new_crc_result[0x0C] = var_crc_result[0x0B];
      new_crc_result[0x0D] = var_crc_result[0x0C];
      new_crc_result[0x0E] = var_crc_result[0x0D];
      new_crc_result[0x0F] = var_crc_result[0x0E] ^ xor_control;
      
    } else {
      new_crc_result = crc_result;
    } 
  }
  crc_result = new_crc_result; 
}
