#include <systemc.h>

/* 
   Verificador de CRC para Mp3
   Daniel Felix Ferber
   daniel.ferber@ic.unicamp.br

   Documentacao:

   Sinais de entrada: 
      reset_n   :   Quando 0 - Mantm o shift register no estado inicial
                    Quando 1 - Armazena o ltimo valor do CRC calculado
      clock     :   Na borda de subida, realiza uma transicao e calcula o 
                    novo CRC que inclui o bit atual (somente se enable = 1)
      enable    :   Quando 0 - No atualiza o shift register
                :   Quando 1 - Atualiza o shift register na subida do clock
      bit_in    :   Um bit de dados para qual vamos calcular o CRC
   Sinais de saida:
      crc_result:   Valor do CRC calculado at o presente momento.
   Nota: 
      Nem todos os frames do mp3 apresentam CRC. Um frame possui dois bytes
      de CRC se e somente se o protection bit do cabecalho for zero. O protection
      bit eh o 16o bit do cabecalho.

      Sao incluidos no calculo do crc:
        - o 3o e 4o byte do cabecalho
        - 17 bytes de dados para mp3 single channel mode
	- 32 bytes de dados para mp3 para os demais modos
*/

#ifndef CRC_H
#define CRC_H

SC_MODULE(crc) {

  sc_in<bool> bit_in;
  sc_in<bool> clock;
  sc_in<bool> enable;
  sc_in<bool> reset_n;

  sc_out< sc_bv<16> > crc_result;

  void crc_check_clock_event();

  SC_CTOR(crc) {
    SC_METHOD(crc_check_clock_event);
      sensitive_pos << clock;
      sensitive_neg << reset_n;
  }
}; 

#endif
