// Generated by CynthVLG 2.5.1  (091704)
// Fri Mar 24 23:02:19 2006
// from crc.cpp
      
/* 
   Verificador de CRC para Mp3
   Daniel Felix Ferber
   daniel.ferber@ic.unicamp.br

   Documentacao:

   Sinais de entrada: 
      reset_n   :   Quando 0 - Mantm o shift register no estado inicial
                    Quando 1 - Armazena o ltimo valor do CRC calculado
      clock     :   Na borda de subida, realiza uma transicao e calcula o 
                    novo CRC que inclui o bit atual (somente se enable = 1)
      enable    :   Quando 0 - No atualiza o shift register
                :   Quando 1 - Atualiza o shift register na subida do clock
      bit_in    :   Um bit de dados para qual vamos calcular o CRC
   Sinais de saida:
      crc_result:   Valor do CRC calculado at o presente momento.
   Nota: 
      Nem todos os frames do mp3 apresentam CRC. Um frame possui dois bytes
      de CRC se e somente se o protection bit do cabecalho for zero. O protection
      bit eh o 16o bit do cabecalho.

      Sao incluidos no calculo do crc:
        - o 3o e 4o byte do cabecalho
        - 17 bytes de dados para mp3 single channel mode
	- 32 bytes de dados para mp3 para os demais modos
*/
module crc(bit_in, clock, enable, reset_n, crc_result);

      input bit_in;
      input clock;
      input enable;
      input reset_n;
      output [15:0] crc_result;
      reg [15:0] crc_result;
  // Temprary signal to be assigned the new value of crc_result
      reg[15:0] new_crc_result;
    // Shift register initial state, ISO spec, page 31 (mpgaudio.doc)
      // G(X) = X^16 + X^15 + X^2 + 17
      // Implementation of Appendix A, page 7, Figure 3-A.9, CRC-Check diagram
      // Temporary signal to allow indexing of out signal bits
      reg[15:0] var_crc_result;
      reg xor_control;

         always @(posedge(clock) or negedge(reset_n))
          begin :crc_check_clock_event
            new_crc_result = 16'd0;
            if (!reset_n) begin
               new_crc_result = /*( [16])*/"1111111111111111";
            end
            else begin
               if (enable) begin
                  var_crc_result = crc_result;
                  xor_control = bit_in ^ var_crc_result[15];
                  new_crc_result[0] = xor_control;
                  new_crc_result[1] = var_crc_result[0];
                  new_crc_result[2] = var_crc_result[1] ^ xor_control;
                  new_crc_result[3] = var_crc_result[2];
                  new_crc_result[4] = var_crc_result[3];
                  new_crc_result[5] = var_crc_result[4];
                  new_crc_result[6] = var_crc_result[5];
                  new_crc_result[7] = var_crc_result[6];
                  new_crc_result[8] = var_crc_result[7];
                  new_crc_result[9] = var_crc_result[8];
                  new_crc_result[10] = var_crc_result[9];
                  new_crc_result[11] = var_crc_result[10];
                  new_crc_result[12] = var_crc_result[11];
                  new_crc_result[13] = var_crc_result[12];
                  new_crc_result[14] = var_crc_result[13];
                  new_crc_result[15] = var_crc_result[14] ^ xor_control;
               end
               else begin
                  new_crc_result = crc_result;
               end
            end
            crc_result <= new_crc_result;
         end


endmodule

