#ifndef ASYNCRAM_H
#define ASYNCRAM_H
#include <assert.h>

#ifdef SIMULATION
#define SIM_SENSITIVE(size)  for(int k=0; k< size; k++) sensitive << buff[k]
#define RAM_TYPE(elem_type) sc_signal<elem_type >
#else
#define SIM_SENSITIVE(size)
#define RAM_TYPE(elem_type) elem_type
#endif

#define DEFINE_ASYNCRAM(size, elem_type, index_type, ram_name) \
        SC_MODULE(ram_name){\
          sc_in<bool>            clk;\
          sc_in<index_type >     addr;\
          sc_in<bool>            we;\
          sc_in<elem_type >      in_data;\
          sc_out<elem_type >     out_data;\
          RAM_TYPE(elem_type)    buff[size];\
          void sync_proc();\
          void async_proc();\
          SC_CTOR(ram_name)\
	   {\
	      SC_METHOD(sync_proc);\
	      sensitive_pos << clk;\
	      SC_METHOD(async_proc);\
              sensitive << addr;\
           }\     
	};\

#ifdef SIMULATION
#define SIM_ASSERT(size)   assert(addr.read() < size)
#else
#define SIM_ASSERT(size)
#endif

#define IMPL_ASYNCRAM(size, ram_name) \
void ram_name::sync_proc(){\
  SIM_ASSERT(size); \
  if(we)\
    buff[addr.read()] = in_data;\
}\
void ram_name::async_proc(){\
  SIM_ASSERT(size); \
  out_data = buff[addr.read()];\
}\
//typedef ram_name SC_CURRENT_USER_MODULE;\

#endif
