/* Constant definitions to be used for all modules */

/* Nomenclatura:

   part of name of constant  |  description
  --------------------------- ----------------------------------
   N_  prefix                |  number of ...
   _SIZE  suffix             |  number of bits to represent ...
   _RAM_LENGTH  suffix       |  number of words in vector ...
   XS_PER_Y                  |  number of X's for each Y
   MB                        |  macroblock
   HOR                       |  horizontal
   VER                       |  vertical
   MV                        |  motion vector
   DMV                       |  delta motion vector
   COEFF                     |  coefficient
   DCOEFF                    |  delta coefficient
   Q                         |  quantizer
   LUM                       |  luminance
   CHROM                     |  crominance
   COMP                      |  component (Y,Cr,Cb or RGB)

.. */

#ifndef MPEG_DEF_H
#define MPEG_DEF_H

#define HOR_PIXELS_PER_BLOCK 8
#define VER_PIXELS_PER_BLOCK HOR_PIXELS_PER_BLOCK
#define PIXELS_PER_BLOCK (HOR_PIXELS_PER_BLOCK*HOR_PIXELS_PER_BLOCK)

#define HOR_PIXELS_PER_MB (2*HOR_PIXELS_PER_BLOCK)
#define VER_PIXELS_PER_MB HOR_PIXELS_PER_MB

#define HOR_COEFFS_PER_BLOCK 8
#define COEFFS_PER_BLOCK PIXELS_PER_BLOCK

#define HOR_LUM_BLOCKS_PER_MB 2
#define VER_LUM_BLOCKS_PER_MB HOR_LUM_BLOCKS_PER_MB
#define LUM_BLOCKS_PER_MB (HOR_LUM_BLOCKS_PER_MB*VER_LUM_BLOCKS_PER_MB)
#define CHROM_BLOCKS_PER_MB 2
#define BLOCKS_PER_MB (LUM_BLOCKS_PER_MB+CHROM_BLOCKS_PER_MB)

#define PIXELS_PER_MB (PIXELS_PER_BLOCK*LUM_BLOCKS_PER_MB)
#define CHROM_COMPS_PER_MB (PIXELS_PER_BLOCK*CHROM_BLOCKS_PER_MB)

#define MBS_PER_LINE 11
#define MBS_PER_COLUMN 9
#define MBS_PER_FRAME (MBS_PER_LINE*MBS_PER_COLUMN)

#define LUM_BLOCKS_PER_LINE (HOR_LUM_BLOCKS_PER_MB*MBS_PER_LINE)
#define LUM_BLOCKS_PER_COLUMN (VER_LUM_BLOCKS_PER_MB*MBS_PER_COLUMN)
#define LUM_BLOCKS_PER_FRAME (LUM_BLOCKS_PER_MB*MBS_PER_FRAME)
#define CROM_BLOCKS_PER_FRAME (CHROM_BLOCKS_PER_MB*MBS_PER_FRAME)
#define BLOCKS_PER_FRAME (LUM_BLOCKS_PER_FRAME+CROM_BLOCKS_PER_FRAME)

#define PIXELS_PER_LINE (HOR_PIXELS_PER_BLOCK*LUM_BLOCKS_PER_LINE)
#define PIXELS_PER_COLUMN (VER_PIXELS_PER_BLOCK*LUM_BLOCKS_PER_COLUMN)
#define PIXELS_PER_FRAME (PIXELS_PER_BLOCK*LUM_BLOCKS_PER_FRAME)
#define PIXEL_COMPS_PER_FRAME (PIXELS_PER_BLOCK*BLOCKS_PER_FRAME)
#define CHROM_COMPS_PER_FRAME (PIXEL_COMPS_PER_FRAME/4)
#define CHROM_COMPS_PER_LINE (PIXELS_PER_LINE/2)
#define CHROM_COMPS_PER_COLUMN (PIXELS_PER_COLUMN/2)

#define COEFFS_PER_BLOCK PIXELS_PER_BLOCK


#define PIXEL_SIZE 8
#define COEFF_SIZE 12
#define MV_SIZE 12
#define DMV_SIZE 6
#define FCODE_SIZE 3
#define Q_SIZE 5

#define HOR_PIXELS_PER_BLOCK_SIZE 3
#define VER_PIXELS_PER_BLOCK_SIZE HOR_PIXELS_PER_BLOCK_SIZE
#define PIXELS_PER_BLOCK_SIZE (VER_PIXELS_PER_BLOCK_SIZE+HOR_PIXELS_PER_BLOCK_SIZE)

#define COEFFS_PER_BLOCK_SIZE PIXELS_PER_BLOCK_SIZE

#define HOR_LUM_BLOCKS_PER_MB_SIZE 1
#define VER_LUM_BLOCKS_PER_MB_SIZE HOR_LUM_BLOCKS_PER_MB_SIZE
#define LUM_BLOCKS_PER_MB_SIZE (HOR_LUM_BLOCKS_PER_MB_SIZE+VER_LUM_BLOCKS_PER_MB_SIZE)
#define CHROM_BLOCKS_PER_MB_SIZE 1

#define LUM_PIXELS_PER_MB_SIZE (PIXELS_PER_BLOCK_SIZE+LUM_BLOCKS_PER_MB_SIZE)
#define CHROM_PIXELS_PER_MB_SIZE (PIXELS_PER_BLOCK_SIZE+CHROM_BLOCKS_PER_MB_SIZE)

#define BLOCKS_PER_MB_SIZE 3
#define MBS_PER_LINE_SIZE 4
#define MBS_PER_COLUMN_SIZE 4
#define MBS_PER_FRAME_SIZE 7

#define LUM_BLOCKS_PER_LINE_SIZE (HOR_LUM_BLOCKS_PER_MB_SIZE+MBS_PER_LINE_SIZE)
#define LUM_BLOCKS_PER_COLUMN_SIZE (VER_LUM_BLOCKS_PER_MB_SIZE+MBS_PER_COLUMN_SIZE)
#define LUM_BLOCKS_PER_FRAME_SIZE (LUM_BLOCKS_PER_MB_SIZE+MBS_PER_FRAME_SIZE)
#define CHROM_BLOCKS_PER_FRAME_SIZE (CHROM_BLOCKS_PER_MB_SIZE+MBS_PER_FRAME_SIZE)
#define BLOCKS_PER_FRAME_SIZE 10

#define PIXELS_PER_LINE_SIZE (HOR_PIXELS_PER_BLOCK_SIZE+LUM_BLOCKS_PER_LINE_SIZE)
#define PIXELS_PER_COLUMN_SIZE (VER_PIXELS_PER_BLOCK_SIZE+LUM_BLOCKS_PER_COLUMN_SIZE)
#define PIXELS_PER_FRAME_SIZE (PIXELS_PER_BLOCK_SIZE+BLOCKS_PER_FRAME_SIZE)
#define PIXEL_COMPS_PER_FRAME_SIZE (PIXELS_PER_BLOCK_SIZE+BLOCKS_PER_FRAME_SIZE)

#define CHROM_COMPS_PER_MB_SIZE    (PIXELS_PER_BLOCK_SIZE + 1) //6+1 = 7

#define COEFFS_PER_BLOCK_SIZE PIXELS_PER_BLOCK_SIZE

#define MV_RAM_SIZE (LUM_BLOCKS_PER_LINE+4)

#define MV_RAM_LENGTH  ((MBS_PER_LINE+1)*4)
#define MV_RAM_LENGTH_SIZE (MBS_PER_LINE_SIZE+2)  // MBS_PER_LINE != 2^n for any n

#define OCP_DATA_WIDTH 8
#define OCP_ADDR_WIDTH 32
#define OCP_IDLE 0
#define OCP_WR 1

#define r read()   // to improve signal/noise ratio
//#define ESPERE(s) while(!(s)) wait();
#endif
