// +FHDR----------------------------------------------------------------------------
// Copyright (c) Brazil-IP / CT-INFO  -  Fenix
// ---------------------------------------------------------------------------------
// FILE NAME :      si.h
// UNIVERITY :      Universidade Federal de Campina Grande
// AUTHOR :         Isaac Maia Pessoa
// AUTHOR S EMAIL : isaacmaia@yahoo.com.br
// ---------------------------------------------------------------------------------
// RELEASE HISTORY 
// VERSION DATE AUTHOR DESCRIPTION 
// 1.0 19/01/04 Isaac inicial 
// ---------------------------------------------------------------------------------
// KEYWORDS : MPEG-4, Inverse Scan,
//            Scan Inverso
// ---------------------------------------------------------------------------------
// PURPOSE :  Receive coefficients of DCDCT and reorder this coefficients using 
//            zig-zag,horizontal or vertical scan.
// ---------------------------------------------------------------------------------
// PARAMETERS
// PARAM NAME RANGE : DESCRIPTION : DEFAULT : UNITS 
// see mpeg_def.h
// ---------------------------------------------------------------------------------
// REUSE ISSUES 
// Reset Strategy : synchronous
// Clock Domains : clk
// Critical Timing : 
// Test Features : 
// Asynchronous I/F : 
// Scan Methodology : 
// Instantiations : none
// Synthesizable (y/n) : y
// Other : 
// -FHDR----------------------------------------------------------------------------

#include "systemc.h"
#include "mpeg_def.h"

SC_MODULE(si) {

       sc_in  <bool> clk;   // system clock
       sc_in  <bool> reset; // system reset

       //States of the state machine
       typedef  enum { IDLE = 0, RECEIVE = 1, RECEIVEBIT = 2, RECEIVEQI = 3,
                       SELECT_SCAN = 4, SEND = 5, FINISH = 6 } state_type;

       //state variable
       sc_signal<sc_uint<3> > state;

       //biststream handshake interface
       sc_signal <bool> intra_block_data ;  //receive flag if intra or inter block
       sc_in     <bool> intra_block;
       sc_in   <bool> bit_valid;
       sc_out  <bool> bit_ready;

       sc_in  <bool> acpred_flag;          //ac predition flag
       sc_signal <bool> acpred_data;       //the received value of the ac prediction flag

       //qi handshake interface
       sc_in  <bool>  dcpred_direction;      //predition direction of DC
       sc_in  <bool>  qi_valid_dc;
       sc_out <bool>  qi_ready_dc;

       //handshake of the coeficients
       sc_in  <bool> in_valid;
       sc_out <bool> out_valid;

       sc_out <bool> in_ready;          //ready to receive new data
       sc_in  <bool> out_ready;         //ready to send data

       // COEFF_SIZE = 12
       // I/O coefficients
       sc_in  <sc_int<12> > qfs; //  receive coefficients
       sc_out <sc_int<12> > pqf; //  send coefficients

       /* internal RAM to be changed for sintetizable module. */
       sc_signal<sc_uint<12> > qfs_ram[64];//ram , COEFFS_PER_BLOCK = 64


       //COEFFS_PER_BLOCK_SIZE = 6
       //A counter for index of array
       sc_signal<sc_uint<6> > count; //limit = 64

       // RAM address
       sc_signal<sc_uint<6> > addr;

       sc_signal< bool >  dcpred_direction_data; //data of predition direction of DC

       // Return a index PQF of array
       sc_uint<6> select_scan (sc_uint<6> count) {

        if( intra_block_data )
          if( acpred_data )
            if( dcpred_direction_data )
              switch ( count ) {      // vertical-scan
                case  0: return  0 ;
                case  8: return  1 ;
                case 16: return  2 ;
                case 24: return  3 ;
                case  1: return  4 ;
                case  9: return  5 ;
                case  2: return  6 ;
                case 10: return  7 ;
                case 17: return  8 ;
                case 25: return  9 ;
                case 32: return  10;
                case 40: return  11;
                case 48: return  12;
                case 56: return  13;
                case 57: return  14;
                case 49: return  15;
                case 41: return  16;
                case 33: return  17;
                case 26: return  18;
                case 18: return  19;
                case 03: return  20;
                case 11: return  21;
                case  4: return  22;
                case 12: return  23;
                case 19: return  24;
                case 27: return  25;
                case 34: return  26;
                case 42: return  27;
                case 50: return  28;
                case 58: return  29;
                case 35: return  30;
                case 43: return  31;
                case 51: return  32;
                case 59: return  33;
                case 20: return  34;
                case 28: return  35;
                case 05: return  36;
                case 13: return  37;
                case 06: return  38;
                case 14: return  39;
                case 21: return  40;
                case 29: return  41;
                case 36: return  42;
                case 44: return  43;
                case 52: return  44;
                case 60: return  45;
                case 37: return  46;
                case 45: return  47;
                case 53: return  48;
                case 61: return  49;
                case 22: return  50;
                case 30: return  51;
                case 07: return  52;
                case 15: return  53;
                case 23: return  54;
                case 31: return  55;
                case 38: return  56;
                case 46: return  57;
                case 54: return  58;
                case 62: return  59;
                case 39: return  60;
                case 47: return  61;
                case 55: return  62;
                default: return  63;
                }

            else
              switch ( count ){   //horizontal-scan    
                case  0 : return   0;
                case  1 : return   1;
                case  2 : return   2;
                case  3 : return   3;
                case  8 : return   4;
                case  9 : return   5;
                case 16 : return   6;
                case 17 : return   7;
                case 10 : return   8;
                case 11 : return   9;
                case  4 : return  10;
                case  5 : return  11;
                case  6 : return  12;
                case  7 : return  13;
                case 15 : return  14;
                case 14 : return  15;
                case 13 : return  16;
                case 12 : return  17;
                case 19 : return  18;
                case 18 : return  19;
                case 24 : return  20;
                case 25 : return  21;
                case 32 : return  22;
                case 33 : return  23;
                case 26 : return  24;
                case 27 : return  25;
                case 20 : return  26;
                case 21 : return  27;
                case 22 : return  28;
                case 23 : return  29;
                case 28 : return  30;
                case 29 : return  31;
                case 30 : return  32;
                case 31 : return  33;
                case 34 : return  34;
                case 35 : return  35;
                case 40 : return  36;
                case 41 : return  37;
                case 48 : return  38;
                case 49 : return  39;
                case 42 : return  40;
                case 43 : return  41;
                case 36 : return  42;
                case 37 : return  43;
                case 38 : return  44;
                case 39 : return  45;
                case 44 : return  46;
                case 45 : return  47;
                case 46 : return  48;
                case 47 : return  49;
                case 50 : return  50;
                case 51 : return  51;
                case 56 : return  52;
                case 57 : return  53;
                case 58 : return  54;
                case 59 : return  55;
                case 52 : return  56;
                case 53 : return  57;
                case 54 : return  58;
                case 55 : return  59;
                case 60 : return  60;
                case 61 : return  61;
                case 62 : return  62;
                default : return  63;	
          }
          
          else
            switch ( count ) {  // zig-zag

                case  0: return  0 ;
                case  1: return  1 ;
                case  8: return  2 ;
                case 16: return  3 ;
                case  9: return  4 ;
                case  2: return  5 ;
                case  3: return  6 ;
                case 10: return  7 ;
                case 17: return  8 ;
                case 24: return  9 ;
                case 32: return 10 ;
                case 25: return 11 ;
                case 18: return 12 ;
                case 11: return 13 ;
                case  4: return 14 ;
                case  5: return 15 ;
                case 12: return 16 ;
                case 19: return 17 ;
                case 26: return 18 ;
                case 33: return 19 ;
                case 40: return 20 ;
                case 48: return 21 ;
                case 41: return 22 ;
                case 34: return 23 ;
                case 27: return 24 ;
                case 20: return 25 ;
                case 13: return 26 ;
                case 06: return 27 ;
                case 07: return 28 ;
                case 14: return 29 ;
                case 21: return 30 ;
                case 28: return 31 ;
                case 35: return 32 ;
                case 42: return 33 ;
                case 49: return 34 ;
                case 56: return 35 ;
                case 57: return 36 ;
                case 50: return 37 ;
                case 43: return 38 ;
                case 36: return 39 ;
                case 29: return 40 ;
                case 22: return 41 ;
                case 15: return 42 ;
                case 23: return 43 ;
                case 30: return 44 ;
                case 37: return 45 ;
                case 44: return 46 ;
                case 51: return 47 ;
                case 58: return 48 ;
                case 59: return 49 ;
                case 52: return 50 ;
                case 45: return 51 ;
                case 38: return 52 ;
                case 31: return 53 ;
                case 39: return 54 ;
                case 46: return 55 ;
                case 53: return 56 ;
                case 60: return 57 ;
                case 61: return 58 ;
                case 54: return 59 ;
                case 47: return 60 ;
                case 55: return 61 ;
                case 62: return 62 ;
                default: return 63 ;
            }
        else
          switch ( count ) {  // zig-zag

                case  0: return  0 ;
                case  1: return  1 ;
                case  8: return  2 ;
                case 16: return  3 ;
                case  9: return  4 ;
                case  2: return  5 ;
                case  3: return  6 ;
                case 10: return  7 ;
                case 17: return  8 ;
                case 24: return  9 ;
                case 32: return 10 ;
                case 25: return 11 ;
                case 18: return 12 ;
                case 11: return 13 ;
                case  4: return 14 ;
                case  5: return 15 ;
                case 12: return 16 ;
                case 19: return 17 ;
                case 26: return 18 ;
                case 33: return 19 ;
                case 40: return 20 ;
                case 48: return 21 ;
                case 41: return 22 ;
                case 34: return 23 ;
                case 27: return 24 ;
                case 20: return 25 ;
                case 13: return 26 ;
                case 06: return 27 ;
                case 07: return 28 ;
                case 14: return 29 ;
                case 21: return 30 ;
                case 28: return 31 ;
                case 35: return 32 ;
                case 42: return 33 ;
                case 49: return 34 ;
                case 56: return 35 ;
                case 57: return 36 ;
                case 50: return 37 ;
                case 43: return 38 ;
                case 36: return 39 ;
                case 29: return 40 ;
                case 22: return 41 ;
                case 15: return 42 ;
                case 23: return 43 ;
                case 30: return 44 ;
                case 37: return 45 ;
                case 44: return 46 ;
                case 51: return 47 ;
                case 58: return 48 ;
                case 59: return 49 ;
                case 52: return 50 ;
                case 45: return 51 ;
                case 38: return 52 ;
                case 31: return 53 ;
                case 39: return 54 ;
                case 46: return 55 ;
                case 53: return 56 ;
                case 60: return 57 ;
                case 61: return 58 ;
                case 54: return 59 ;
                case 47: return 60 ;
                case 55: return 61 ;
                case 62: return 62 ;
                default: return 63 ;
            }
          
       }//select_scan
  
  void proc_ram()
  {
      if(state.read()==IDLE && in_valid ) qfs_ram[addr.read()] =  (sc_uint<COEFF_SIZE>)qfs.read(); 
       else 
         if ( (state.read()==SEND && out_ready) || state.read()==FINISH )   pqf = (sc_int<COEFF_SIZE>)qfs_ram[ addr.read() ].read();
          else pqf = 0; 
  }
  
  void proc () {

    if (reset) {
      count = 0;
      addr = 0;
      in_ready = 1;
      out_valid = 0;
      bit_ready = 0;
      qi_ready_dc = 0;
      state = IDLE;
      intra_block_data = 0;
      qi_ready_dc = 0;
      acpred_data = 0;
      dcpred_direction_data = 0;
      
    }
    else {

      switch( state.read() ) {
      case IDLE:
        if ( in_valid ){  // ciclo de escrita na ram (ver proc_ram)
          count = count.read() + 1;
          in_ready = 0;
          state = RECEIVE;
        }
        break;

      case RECEIVE:
        if (count.read() == 0) { //count = 64 => overflow => count = 0
          state = RECEIVEBIT;
          bit_ready = 1;
        }
        else {
          if ( ! in_valid ) {
            in_ready = 1;
            addr = count;
            state = IDLE;
          }
        }
        break;

      case RECEIVEBIT:
      
        if ( bit_valid ) {
          bit_ready = 0;
          qi_ready_dc = 1;
          intra_block_data = intra_block.read();
          acpred_data = acpred_flag.read();
          state = RECEIVEQI;        
        }else
            state = RECEIVEBIT;       
        break;
       
      case RECEIVEQI:
        if ( qi_valid_dc ) {          
           qi_ready_dc = 0;
           dcpred_direction_data = dcpred_direction.read();
           state = SELECT_SCAN;
        }else
          state = RECEIVEQI;
        break;

      case SELECT_SCAN:
       
        //Scan select of bitstream
        addr = select_scan(count.read());
        state = SEND;
        break;


      case SEND:
        if ( out_ready  ){
          out_valid = 1;               
          count = count.read() + 1;               
          state = FINISH;
        }else {          
          state = SEND;
        }
        
        break;

      case FINISH:
        if ( count.read() == 0 ) { //count = 64 => overflow => count = 0
          if ( !out_ready ){
            out_valid = 0;
            addr = count;
            in_ready = 1;
            state = IDLE;
          }
        }
        else {
          if ( !out_ready ) {
            out_valid = 0;
            state = SELECT_SCAN;
          }
        }
        break;

      default: state = IDLE;

      }//switch
    }//else
  }//receive

  SC_CTOR( si ) {
    SC_METHOD( proc ); sensitive_pos << clk;
    SC_METHOD( proc_ram ); sensitive_pos << clk;

  }

};
  
  
    
